@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -268,16 +268,10 @@ var hintBox = {
 				: jQuery(this);
 
 			if (e.type === 'keydown') {
-				if (e.which !== 13) {
+				if (e.key !== ' ' && e.key !== 'Enter') {
 					return;
 				}
 
-				const offset = $target.offset(),
-					w = jQuery(window);
-
-				// Emulate a click on the left middle point of the target.
-				e.clientX = offset.left - w.scrollLeft();
-				e.clientY = offset.top - w.scrollTop() + ($target.height() / 2);
 				e.preventDefault();
 			}
 
@@ -400,7 +394,6 @@ var hintBox = {
 					width: '',
 					height: '',
 					top: '',
-					right: '',
 					left: ''
 				});
 
@@ -512,7 +505,28 @@ var hintBox = {
 			jQuery(target).data('return-control', jQuery(e.target));
 
 			if (resizeAfterLoad) {
+				const resetSize = () => {
+					for (const css_property of ['width', 'height']) {
+						target.hintBoxItem[0].style[css_property] = null;
+					}
+				};
+
+				resetSize();
+
+				const preloader = document.createElement('div');
+				preloader.id = 'hintbox-preloader';
+				preloader.className = 'is-loading hintbox-preloader';
+
+				target.hintBoxItem[0].append(preloader);
+
+				hintText[0].style.display = 'none';
+
 				hintText.one('load', function(e) {
+					resetSize();
+
+					preloader.remove();
+					hintText[0].style.display = null;
+
 					hintBox.positionElement(e, target, target.hintBoxItem);
 				});
 			}
@@ -569,75 +583,107 @@ var hintBox = {
 	},
 
 	positionElement: function(e, target, $elem) {
-		if (e.clientX) {
-			target.clientX = e.clientX;
-			target.clientY = e.clientY;
-		}
+		const hint = $elem[0];
+		const host = hint.offsetParent;
+		const host_rect = host.getBoundingClientRect();
+		const event_offset = 10;
+		const css = {
+			width: null,
+			height: null,
+			top: null,
+			left: null
+		};
 
-		var $host = $elem.offsetParent(),
-			host_offset = $host.offset(),
-			// Usable area relative to host.
-			host_x_min = $host.scrollLeft(),
-			host_x_max = Math.min($host[0].scrollWidth,
-				$(window).width() + $(window).scrollLeft() - host_offset.left + $host.scrollLeft()
-			) - 1,
-			host_y_min = $host.scrollTop(),
-			host_y_max = Math.min($host[0].scrollHeight,
-				$(window).height() + $(window).scrollTop() - host_offset.top + $host.scrollTop()
-			) - 1,
-			// Event coordinates relative to host.
-			event_x = target.clientX - host_offset.left + $host.scrollLeft(),
-			event_y = target.clientY - host_offset.top + $host.scrollTop(),
-			event_offset = 10,
-			// Hint box width and height.
-			hint_width = $elem.outerWidth(),
-			hint_height = $elem.outerHeight(),
-			/*
-				Fix popup width and height since browsers will tend to reduce the size of the popup, if positioned further
-				than the width of window when horizontal scrolling is active.
-			*/
-			css = {
-				width: null,
-				height: null,
-				top: null,
-				right: null,
-				left: null
-			};
+		const resetPosition = () => {
+			Object.keys(css).forEach(key => {
+				css[key] = null;
+				hint.style[key] = null;
+			});
+		};
 
-		if ($host[0].clientWidth > hint_width) {
-			css.width = Math.ceil($elem.width());
-			css.height = Math.ceil($elem.height());
-		}
+		resetPosition();
 
-		if (event_x + event_offset + hint_width <= host_x_max) {
-			css.left = event_x + event_offset;
-		}
-		else {
-			css.right = -$host.scrollLeft() || 0;
-		}
+		let host_client_width = host.clientWidth;
+		let host_client_height = host.clientHeight;
 
-		if (event_y + event_offset + hint_height <= host_y_max) {
-			css.top = event_y + event_offset;
-		}
-		else if (event_y - event_offset - hint_height >= host_y_min) {
-			css.top = event_y - event_offset - hint_height;
-		}
-		else {
-			css.top = Math.max(host_y_min, Math.min(host_y_max - hint_height, event_y + event_offset));
-
-			if (css.right !== null) {
-				css.right = null;
-
-				css.left = ((event_x - event_offset - hint_width >= host_x_min)
-					? event_x - event_offset - hint_width
-					: event_x + event_offset
-				);
+		do {
+			// Check if client size has decreased (because of scrollbars).
+			if (host_client_width > host.clientWidth) {
+				host.style.overflowY = 'scroll';
+			}
+			if (host_client_height > host.clientHeight) {
+				host.style.overflowX = 'scroll';
 			}
-		}
 
-		for (const [key, value] of Object.entries(css)) {
-			$elem[0].style[key] = value !== null ? `${value}px` : null;
+			host_client_width = host.clientWidth;
+			host_client_height = host.clientHeight;
+
+			resetPosition();
+
+			const scrollbar_vertical_width = host.offsetWidth - host.clientWidth;
+			const scrollbar_horizontal_height = host.offsetHeight - host.clientHeight;
+
+			// Usable area relative to host.
+			const host_x_min = host.scrollLeft;
+			const host_x_max = Math.min(host.scrollWidth, host_rect.width - scrollbar_vertical_width + host_x_min);
+			const host_y_min = host.scrollTop;
+			const host_y_max = Math.min(host.scrollHeight, host_rect.height - scrollbar_horizontal_height + host_y_min);
+
+			// Hint size.
+			const hint_rect = hint.getBoundingClientRect();
+			const hint_computed_style = getComputedStyle(hint);
+
+			/*
+				Fix popup width and height since browsers will tend to reduce the size of the popup,
+				if positioned further than the width of window when horizontal scrolling is active.
+			*/
+			css.width = Math.ceil(parseFloat(hint_computed_style.width));
+			css.height = Math.ceil(parseFloat(hint_computed_style.height));
+
+			// Event coordinates relative to host.
+			if (target.event_x === undefined) {
+				let client_x, client_y;
+
+				if (e.clientX !== undefined) {
+					client_x = e.clientX;
+					client_y = e.clientY;
+				}
+				else {
+					const $target = jQuery(target);
+					const offset = $target.offset();
+
+					client_x = offset.left;
+					client_y = offset.top + $target.height() / 2;
+				}
+
+				target.event_x = client_x - host_rect.left + host_x_min;
+				target.event_y = client_y - host_rect.top + host_y_min;
+			}
+
+			css.left = target.event_x + event_offset + hint_rect.width <= host_x_max
+				? target.event_x + event_offset
+				: host_x_max - hint_rect.width;
+
+			// Hint fits under event.
+			if (target.event_y + event_offset + hint_rect.height <= host_y_max) {
+				css.top = target.event_y + event_offset;
+			}
+			// Hint fits above event.
+			else if (target.event_y - event_offset - hint_rect.height >= host_y_min) {
+				css.top = target.event_y - event_offset - hint_rect.height;
+			}
+			// Hint fits neither under nor above event - then show it under event.
+			else {
+				css.top = target.event_y + event_offset;
+			}
+
+			// Assign css rules to hint.
+			Object.entries(css).forEach(([key, value]) => hint.style[key] = value !== null ? `${value}px` : null);
 		}
+		while (host_client_width > host.clientWidth || host_client_height > host.clientHeight);
+
+		host.style.overflowX = null;
+		host.style.overflowY = null;
 	},
 
 	hideHint: function(target, hideStatic) {
@@ -653,6 +699,9 @@ var hintBox = {
 	},
 
 	deleteHint: function(target, do_focus_target = true) {
+		delete target.event_x;
+		delete target.event_y;
+
 		if (typeof target.hintboxid !== 'undefined') {
 			jQuery(target).removeAttr('data-expanded');
 			removeFromOverlaysStack(target.hintboxid, do_focus_target);
@@ -730,30 +779,31 @@ var hintBox = {
 function ApiCall(method, params, id = 1) {
 	return fetch(new Curl('api_jsonrpc.php').getUrl(), {
 		method: 'POST',
-		headers: {
-			'Content-Type': 'application/json'
-		},
+		headers: {'Content-Type': 'application/json'},
 		credentials: 'same-origin',
-		body: JSON.stringify({
-			jsonrpc: '2.0',
-			method,
-			params,
-			id
-		}),
-	}).then((response) => response.json());
+		body: JSON.stringify({jsonrpc: '2.0', method, params, id}),
+	})
+		.then(response => response.json())
+		.then(response => {
+			if ('error' in response) {
+				throw new Error(response.error.message, {
+					cause: { code: response.error.code, data: response.error.data }
+				});
+			}
+
+			return response;
+		});
 }
 
 /**
  * Section collapse toggle.
  *
- * @param {string}      id
- * @param {string|null} profile_idx  If not null, stores state in profile.
+ * @param {HTMLButtonElement} toggle
+ * @param {string} profile_idx  If not null, stores state in profile.
  */
-function toggleSection(id, profile_idx) {
-	const section = document.getElementById(id);
-	const toggle = section.querySelector('.section-toggle');
-
-	let is_collapsed = section.classList.contains(ZBX_STYLE_COLLAPSED);
+function toggleSection(toggle, profile_idx = '') {
+	const section = toggle.closest(`.${ZBX_STYLE_COLLAPSIBLE}`);
+	const is_collapsed = section.classList.contains(ZBX_STYLE_COLLAPSED);
 
 	section.classList.toggle(ZBX_STYLE_COLLAPSED, !is_collapsed);
 
@@ -761,6 +811,8 @@ function toggleSection(id, profile_idx) {
 	toggle.classList.toggle(ZBX_ICON_CHEVRON_UP, is_collapsed);
 	toggle.setAttribute('title', is_collapsed ? t('S_COLLAPSE') : t('S_EXPAND'));
 
+	section.dispatchEvent(new CustomEvent(!is_collapsed ? 'collapse' : 'expand'));
+
 	if (profile_idx !== '') {
 		updateUserProfile(profile_idx, is_collapsed ? '1' : '0', []);
 	}
@@ -1119,15 +1171,25 @@ function getConditionFormula(conditions, evalType) {
 	 */
 	function removeRow(table, row, options) {
 		if (options.remove_next_sibling) {
+			removeErrorContainers(row.next());
 			row.next().remove();
 		}
 
+		removeErrorContainers(row);
 		row.remove();
 
 		table.trigger('tableupdate.dynamicRows', options);
 		table.trigger('afterremove.dynamicRows', options);
 	}
 
+	function removeErrorContainers($row) {
+		$('[data-error-container]', $row).each(function () {
+			if ($('#' + $(this).data('errorContainer'), $row.next()).length) {
+				$row.next().remove();
+			}
+		});
+	}
+
 	/**
 	 * Disables the given row.
 	 *
@@ -1158,6 +1220,11 @@ document.addEventListener('DOMContentLoaded', () => {
 	if (typeof CSoftwareVersionCheck !== 'undefined') {
 		ZABBIX.SoftwareVersionCheck = new CSoftwareVersionCheck();
 	}
+
+	// Popup manager initialization (no object creation).
+
+	ZABBIX.PopupManager = CPopupManager;
+	ZABBIX.PopupManager.init();
 });
 
 window.addEventListener('load', () => {
